<div id="widgetvisualization-{{widget.pk}}" rel="{{widget.pk}}" class="widgetvisualizationdialog" title="{{widget.name}} visualization" width="800px" height="600px">



    <iframe src="{% url 'nltk get adc index' widget_id=widget.pk  %}" width="100%" height="100%"></iframe>

    <div style="position:absolute; bottom:5px; left:10px;">
        <a target="_blank" style="color: gray;" href="{% url 'nltk get adc index' widget_id=widget.pk narrow_doc=''%}">Open corpus view in new window</a>
    </div>




</div>
