from lib.bag_of_words import *
from lib.classification import *
#from lib.clustering import *
from lib.document_corpus import *
#from lib.similarity_matrix import *
from lib.part_of_speech_tagging import *
from lib.stop_word_removal import *
from lib.stemming import *
from lib.textual_data_in_out import *
from lib.tokenization import *
from lib.chunking import *