{% load static %}
{% if data.1 != "Chi square feature evaluation" %}
    <div id="widgetvisualization-{{widget.pk}}" rel="{{widget.pk}}" class="widgetvisualizationdialog" title="{{widget.name}} visualization" width="400px" height="500px">
{% else %}
    <div id="widgetvisualization-{{widget.pk}}" rel="{{widget.pk}}" class="widgetvisualizationdialog" title="{{widget.name}} visualization" width="500px" height="500px">
{% endif %}
    <div style="position:absolute; top:5px; left:75px;">
        <p style="padding-top:5px; padding-bottom:10px; padding-left:25px"><b>{{data.1}}</b></p>
        <ul>
        {% if data.1 != "Chi square feature evaluation" %}
            {% for i,j in data.0 %}

                <li style="padding-top:3px;">
                    <p style="width:200px; float:left;"><b>{{i}}</b></p>
                    <p style="width:100px; float:left;">{{j|floatformat:"2"}}</p>
                </li>

            {% endfor %}
        {% else %}
            <p style="width:200px; float:left; padding-top:5px; padding-bottom:10px;"><b>Feature</b></p>
            <p style="width:100px; float:left; padding-top:5px; padding-bottom:10px;">Chi score</p>
            <p style="width:100px; float:left; padding-top:5px; padding-bottom:10px;">P-value</p>
            {% for i,j,k in data.0 %}
                <li style="padding-top:3px;">
                     <p style="width:200px; float:left;"><b>{{i}}</b></p>
                     <p style="width:100px; float:left;">{{j}}</p>
                     <p style="width:100px; float:left;">{{k}}</p>
                </li>

            {% endfor %}
        {% endif %}
        </ul>
    </div>
</div>