body {font-family:helvetica;}
#DocumentTitle {color:#3C64B6;background-color: #EEEEEE;font-weight:bold;font-size:1.5em;padding-bottom:1em;padding-top:1em;padding-left:0.3em;}
#TextLayer {position:absolute;left:450px;top:60px;width:60%;height:auto;padding-left:3em;border-left:1px #3C64B6 solid;z-index:1;}
#ListLayer {position:relative;left:50px;top:60px;width:30%;height:auto;z-index:1;}
#DocumentFeaturesLayer {position:absolute;left:50px;width:300px;height:auto;padding-left:2.1em;padding-top:1.5em;z-index:1;word-wrap:break-word;}
#h3Tree {padding-left:4.5em;}
#backlink {text-align:right;width:95%;}
#backlinkA {color:Gray}
ul, li, ul li a {list-style: none;width:100%;}
.documentText, #DocumentText {line-height: 120%;}

h3 {color: #3C64B6; font-weight:bold;}
a:link, a:visited, a:active, a:hover { color: #3C64B6; outline: 0; }
.tooltip {background-color:#26517E;border:1px solid #fff;padding:10px 15px;display:none;color:#fff;text-align:left;font-size:15px;font-weight:bolder;z-index: 8000;-moz-box-shadow:0 0 10px #000;-webkit-box-shadow:0 0 10px #000;}