body {margin-left: 15%; margin-right: 10%; margin-top: 5%; font-family:helvetica; }
#documentList {position:absolute;top:60px;width:600px;left:400px;height:auto;border-left:1px #3C64B6 solid;padding-left:2em;z-index:2;}
#corpusFeatures {position:absolute;left:50px;top:60px;padding-top:1em;width:300px;height:auto;z-index:1;word-wrap:break-word;font-size:0.9em;}
.documentTitle {font-weight:bold; font-size:1.2em; margin-top: 1.5em; margin-bottom: 0.5em; }
.statistics {margin-top: 0.3em; margin-bottom: 0em; width: 50em; font-size:0.8em; color:#CD3700;}
.documentText {margin-top: 0em; margin-bottom: 0em; width: 50em; font-size:1em;line-height: 120%;}
h2 {color: #004D99; font-weight:bold}

#backlinkA {color:Gray}
#ListLayer {position:relative;left:50px;top:60px;width:30%;height:auto;z-index:1;}